@@ -492,6 +492,7 @@ class CloudRegion(object):
                     region_name=self.region_name,
                 )
                 network_endpoint = network_adapter.get_endpoint()
+                network_adapter.task_manager.stop()
                 if not network_endpoint.rstrip().rsplit('/')[-1] == 'v2.0':
                     if not network_endpoint.endswith('/'):
                         network_endpoint += '/'
@@ -595,7 +596,7 @@ class CloudRegion(object):
         if not endpoint:
             self.log.warning(
                 "Keystone catalog entry not found ("
-                "service_type=%s,service_name=%s"
+                "service_type=%s,service_name=%s,"
                 "interface=%s,region_name=%s)",
                 service_type,
                 service_name,
