@@ -11,6 +11,7 @@
 # under the License.
 
 import mock
+import six
 from openstack.tests.unit import base
 
 from openstack.compute.v2 import server
@@ -72,10 +73,9 @@ class TestServer(base.TestCase):
         self.assertEqual('server', sot.resource_key)
         self.assertEqual('servers', sot.resources_key)
         self.assertEqual('/servers', sot.base_path)
-        self.assertEqual('compute', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
@@ -84,7 +84,7 @@ class TestServer(base.TestCase):
                               "name": "name",
                               "status": "status",
                               "host": "host",
-                              "all_tenants": "all_tenants",
+                              "all_projects": "all_tenants",
                               "changes_since": "changes-since",
                               "limit": "limit",
                               "marker": "marker",
@@ -93,9 +93,9 @@ class TestServer(base.TestCase):
                               "reservation_id": "reservation_id",
                               "project_id": "project_id",
                               "tags": "tags",
-                              "tags_any": "tags-any",
+                              "any_tags": "tags-any",
                               "not_tags": "not-tags",
-                              "not_tags_any": "not-tags-any",
+                              "not_any_tags": "not-tags-any",
                               "is_deleted": "deleted",
                               "ipv4_address": "ip",
                               "ipv6_address": "ip6",
@@ -154,10 +154,9 @@ class TestServer(base.TestCase):
         self.assertEqual('server', sot.resource_key)
         self.assertEqual('servers', sot.resources_key)
         self.assertEqual('/servers/detail', sot.base_path)
-        self.assertEqual('compute', sot.service.service_type)
         self.assertFalse(sot.allow_create)
-        self.assertFalse(sot.allow_get)
-        self.assertFalse(sot.allow_update)
+        self.assertFalse(sot.allow_fetch)
+        self.assertFalse(sot.allow_commit)
         self.assertFalse(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
@@ -193,7 +192,7 @@ class TestServer(base.TestCase):
         body = {"changePassword": {"adminPass": "a"}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_reboot(self):
         sot = server.Server(**EXAMPLE)
@@ -204,7 +203,7 @@ class TestServer(base.TestCase):
         body = {"reboot": {"type": "HARD"}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_force_delete(self):
         sot = server.Server(**EXAMPLE)
@@ -215,7 +214,7 @@ class TestServer(base.TestCase):
         body = {'forceDelete': None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_rebuild(self):
         sot = server.Server(**EXAMPLE)
@@ -246,7 +245,7 @@ class TestServer(base.TestCase):
         }
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_rebuild_minimal(self):
         sot = server.Server(**EXAMPLE)
@@ -270,7 +269,7 @@ class TestServer(base.TestCase):
         }
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_resize(self):
         sot = server.Server(**EXAMPLE)
@@ -281,7 +280,7 @@ class TestServer(base.TestCase):
         body = {"resize": {"flavorRef": "2"}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_confirm_resize(self):
         sot = server.Server(**EXAMPLE)
@@ -292,7 +291,7 @@ class TestServer(base.TestCase):
         body = {"confirmResize": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_revert_resize(self):
         sot = server.Server(**EXAMPLE)
@@ -303,7 +302,7 @@ class TestServer(base.TestCase):
         body = {"revertResize": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_create_image(self):
         sot = server.Server(**EXAMPLE)
@@ -316,7 +315,7 @@ class TestServer(base.TestCase):
         body = {"createImage": {'name': name, 'metadata': metadata}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_create_image_minimal(self):
         sot = server.Server(**EXAMPLE)
@@ -328,7 +327,7 @@ class TestServer(base.TestCase):
         body = {"createImage": {'name': name}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_add_security_group(self):
         sot = server.Server(**EXAMPLE)
@@ -339,7 +338,7 @@ class TestServer(base.TestCase):
         body = {"addSecurityGroup": {"name": "group"}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_remove_security_group(self):
         sot = server.Server(**EXAMPLE)
@@ -350,7 +349,7 @@ class TestServer(base.TestCase):
         body = {"removeSecurityGroup": {"name": "group"}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_reset_state(self):
         sot = server.Server(**EXAMPLE)
@@ -361,7 +360,7 @@ class TestServer(base.TestCase):
         body = {"os-resetState": {"state": 'active'}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_add_fixed_ip(self):
         sot = server.Server(**EXAMPLE)
@@ -373,7 +372,7 @@ class TestServer(base.TestCase):
         body = {"addFixedIp": {"networkId": "NETWORK-ID"}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_remove_fixed_ip(self):
         sot = server.Server(**EXAMPLE)
@@ -385,7 +384,7 @@ class TestServer(base.TestCase):
         body = {"removeFixedIp": {"address": "ADDRESS"}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_add_floating_ip(self):
         sot = server.Server(**EXAMPLE)
@@ -397,7 +396,7 @@ class TestServer(base.TestCase):
         body = {"addFloatingIp": {"address": "FLOATING-IP"}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_add_floating_ip_with_fixed_addr(self):
         sot = server.Server(**EXAMPLE)
@@ -410,7 +409,7 @@ class TestServer(base.TestCase):
                                   "fixed_address": "FIXED-ADDR"}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_remove_floating_ip(self):
         sot = server.Server(**EXAMPLE)
@@ -422,7 +421,7 @@ class TestServer(base.TestCase):
         body = {"removeFloatingIp": {"address": "I-AM-FLOATING"}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_backup(self):
         sot = server.Server(**EXAMPLE)
@@ -435,7 +434,7 @@ class TestServer(base.TestCase):
                                  "rotation": 1}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_pause(self):
         sot = server.Server(**EXAMPLE)
@@ -447,7 +446,7 @@ class TestServer(base.TestCase):
         body = {"pause": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_unpause(self):
         sot = server.Server(**EXAMPLE)
@@ -459,7 +458,7 @@ class TestServer(base.TestCase):
         body = {"unpause": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_suspend(self):
         sot = server.Server(**EXAMPLE)
@@ -471,7 +470,7 @@ class TestServer(base.TestCase):
         body = {"suspend": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_resume(self):
         sot = server.Server(**EXAMPLE)
@@ -483,7 +482,7 @@ class TestServer(base.TestCase):
         body = {"resume": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_lock(self):
         sot = server.Server(**EXAMPLE)
@@ -495,7 +494,7 @@ class TestServer(base.TestCase):
         body = {"lock": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_unlock(self):
         sot = server.Server(**EXAMPLE)
@@ -507,7 +506,7 @@ class TestServer(base.TestCase):
         body = {"unlock": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_rescue(self):
         sot = server.Server(**EXAMPLE)
@@ -519,7 +518,7 @@ class TestServer(base.TestCase):
         body = {"rescue": {}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_rescue_with_options(self):
         sot = server.Server(**EXAMPLE)
@@ -532,7 +531,7 @@ class TestServer(base.TestCase):
                            'rescue_image_ref': 'IMG-ID'}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_unrescue(self):
         sot = server.Server(**EXAMPLE)
@@ -544,7 +543,7 @@ class TestServer(base.TestCase):
         body = {"unrescue": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_evacuate(self):
         sot = server.Server(**EXAMPLE)
@@ -556,7 +555,7 @@ class TestServer(base.TestCase):
         body = {"evacuate": {}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_evacuate_with_options(self):
         sot = server.Server(**EXAMPLE)
@@ -570,7 +569,7 @@ class TestServer(base.TestCase):
                              'force': True}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_start(self):
         sot = server.Server(**EXAMPLE)
@@ -582,7 +581,7 @@ class TestServer(base.TestCase):
         body = {"os-start": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_stop(self):
         sot = server.Server(**EXAMPLE)
@@ -594,7 +593,7 @@ class TestServer(base.TestCase):
         body = {"os-stop": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_shelve(self):
         sot = server.Server(**EXAMPLE)
@@ -606,7 +605,7 @@ class TestServer(base.TestCase):
         body = {"shelve": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_unshelve(self):
         sot = server.Server(**EXAMPLE)
@@ -618,7 +617,7 @@ class TestServer(base.TestCase):
         body = {"unshelve": None}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_migrate(self):
         sot = server.Server(**EXAMPLE)
@@ -631,7 +630,7 @@ class TestServer(base.TestCase):
 
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
     def test_get_console_output(self):
         sot = server.Server(**EXAMPLE)
@@ -643,7 +642,7 @@ class TestServer(base.TestCase):
         body = {'os-getConsoleOutput': {}}
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
         res = sot.get_console_output(self.sess, length=1)
 
@@ -653,23 +652,143 @@ class TestServer(base.TestCase):
 
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
 
-    def test_live_migrate(self):
+    def test_live_migrate_no_force(self):
         sot = server.Server(**EXAMPLE)
 
-        res = sot.live_migrate(self.sess, host='HOST2', force=False)
+        class FakeEndpointData(object):
+            min_microversion = None
+            max_microversion = None
+        self.sess.get_endpoint_data.return_value = FakeEndpointData()
+
+        ex = self.assertRaises(
+            ValueError,
+            sot.live_migrate,
+            self.sess, host='HOST2', force=False, block_migration=False)
+        self.assertIn(
+            "Live migration on this cloud implies 'force'",
+            six.text_type(ex))
+
+    def test_live_migrate_no_microversion_force_true(self):
+        sot = server.Server(**EXAMPLE)
+
+        class FakeEndpointData(object):
+            min_microversion = None
+            max_microversion = None
+        self.sess.get_endpoint_data.return_value = FakeEndpointData()
+
+        res = sot.live_migrate(
+            self.sess, host='HOST2', force=True, block_migration=True,
+            disk_over_commit=True)
 
         self.assertIsNone(res)
         url = 'servers/IDENTIFIER/action'
         body = {
-            "os-migrateLive": {
-                "host": 'HOST2',
-                "block_migration": "auto",
-                "force": False
+            'os-migrateLive': {
+                'host': 'HOST2',
+                'disk_over_commit': True,
+                'block_migration': True
             }
         }
 
         headers = {'Accept': ''}
         self.sess.post.assert_called_with(
-            url, json=body, headers=headers)
+            url, json=body, headers=headers, microversion=None)
+
+    def test_live_migrate_25(self):
+        sot = server.Server(**EXAMPLE)
+
+        class FakeEndpointData(object):
+            min_microversion = '2.1'
+            max_microversion = '2.25'
+        self.sess.get_endpoint_data.return_value = FakeEndpointData()
+
+        res = sot.live_migrate(
+            self.sess, host='HOST2', force=True, block_migration=False)
+
+        self.assertIsNone(res)
+        url = 'servers/IDENTIFIER/action'
+        body = {
+            "os-migrateLive": {
+                'block_migration': False,
+                'host': 'HOST2',
+            }
+        }
+
+        headers = {'Accept': ''}
+        self.sess.post.assert_called_with(
+            url, json=body, headers=headers, microversion='2.25')
+
+    def test_live_migrate_25_default_block(self):
+        sot = server.Server(**EXAMPLE)
+
+        class FakeEndpointData(object):
+            min_microversion = '2.1'
+            max_microversion = '2.25'
+        self.sess.get_endpoint_data.return_value = FakeEndpointData()
+
+        res = sot.live_migrate(
+            self.sess, host='HOST2', force=True, block_migration=None)
+
+        self.assertIsNone(res)
+        url = 'servers/IDENTIFIER/action'
+        body = {
+            "os-migrateLive": {
+                'block_migration': 'auto',
+                'host': 'HOST2',
+            }
+        }
+
+        headers = {'Accept': ''}
+        self.sess.post.assert_called_with(
+            url, json=body, headers=headers, microversion='2.25')
+
+    def test_live_migrate_30(self):
+        sot = server.Server(**EXAMPLE)
+
+        class FakeEndpointData(object):
+            min_microversion = '2.1'
+            max_microversion = '2.30'
+        self.sess.get_endpoint_data.return_value = FakeEndpointData()
+
+        res = sot.live_migrate(
+            self.sess, host='HOST2', force=False, block_migration=False)
+
+        self.assertIsNone(res)
+        url = 'servers/IDENTIFIER/action'
+        body = {
+            'os-migrateLive': {
+                'block_migration': False,
+                'host': 'HOST2'
+            }
+        }
+
+        headers = {'Accept': ''}
+        self.sess.post.assert_called_with(
+            url, json=body, headers=headers, microversion='2.30')
+
+    def test_live_migrate_30_force(self):
+        sot = server.Server(**EXAMPLE)
+
+        class FakeEndpointData(object):
+            min_microversion = '2.1'
+            max_microversion = '2.30'
+        self.sess.get_endpoint_data.return_value = FakeEndpointData()
+
+        res = sot.live_migrate(
+            self.sess, host='HOST2', force=True, block_migration=None)
+
+        self.assertIsNone(res)
+        url = 'servers/IDENTIFIER/action'
+        body = {
+            'os-migrateLive': {
+                'block_migration': 'auto',
+                'host': 'HOST2',
+                'force': True,
+            }
+        }
+
+        headers = {'Accept': ''}
+        self.sess.post.assert_called_with(
+            url, json=body, headers=headers, microversion='2.30')
